<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\DataFixtures\Factory;

use Dedi\SyliusEcotaxPlugin\Entity\EcotaxTypeInterface;
use Faker\Factory;
use Faker\Generator;
use Sylius\Bundle\CoreBundle\Fixture\Factory\AbstractExampleFactory;
use Sylius\Bundle\CoreBundle\Fixture\Factory\ExampleFactoryInterface;
use Sylius\Component\Core\Formatter\StringInflector;
use Sylius\Component\Locale\Model\LocaleInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\OptionsResolver\Options;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Webmozart\Assert\Assert;

class EcotaxTypeFactory extends AbstractExampleFactory implements ExampleFactoryInterface
{
    private Generator $faker;

    /**
     * @param FactoryInterface<EcotaxTypeInterface> $ecotaxTypeFactory
     * @param RepositoryInterface<EcotaxTypeInterface> $localeRepository
     */
    public function __construct(
        private FactoryInterface $ecotaxTypeFactory,
        private RepositoryInterface $localeRepository,
    ) {
        $this->faker = Factory::create();
    }

    public function create(array $options = []): EcotaxTypeInterface
    {
        /** @var EcotaxTypeInterface $ecotaxType */
        $ecotaxType = $this->ecotaxTypeFactory->createNew();
        $ecotaxType->setCode($options['code']);

        $this->createTranslations($ecotaxType, $options);

        return $ecotaxType;
    }

    protected function configureOptions(OptionsResolver $resolver): void
    {
        $resolver
            ->setDefault('label', function (Options $options): string {
                /** @var string $words */
                $words = $this->faker->words(3, true);

                return $words;
            })
            ->setDefault('code', function (Options $options): string {
                Assert::string($options['name']);

                return StringInflector::nameToCode($options['name']);
            })
        ;
    }

    private function createTranslations(EcotaxTypeInterface $ecotaxType, array $options): void
    {
        foreach ($this->getLocales() as $localeCode) {
            $ecotaxType->setCurrentLocale($localeCode);
            $ecotaxType->setFallbackLocale($localeCode);

            $ecotaxType->setLabel($options['label']);
        }
    }

    private function getLocales(): iterable
    {
        /** @var LocaleInterface[] $locales */
        $locales = $this->localeRepository->findAll();
        foreach ($locales as $locale) {
            yield $locale->getCode();
        }
    }
}
