<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Model;

use Sylius\Component\Core\Model\AdjustmentInterface;

trait EcotaxAdjustmentsAwareTrait
{
    public function getEcotaxTotal(): int
    {
        return $this->getAdjustmentsTotalRecursively(EcotaxAdjustmentsAwareInterface::ADJUSTMENT_TYPE_ECOTAX);
    }

    public function getEcotaxesTotals(): array
    {
        /** @var AdjustmentInterface[] $ecotaxes */
        $ecotaxes = $this->getAdjustmentsRecursively(EcotaxAdjustmentsAwareInterface::ADJUSTMENT_TYPE_ECOTAX);

        $totals = [];
        foreach ($ecotaxes as $ecotax) {
            if (!array_key_exists($ecotax->getLabel(), $totals)) {
                $totals[$ecotax->getLabel()] = [
                    'label' => $ecotax->getDetails()['label'],
                    'amount' => 0,
                ];
            }

            $totals[$ecotax->getLabel()]['amount'] += $ecotax->getDetails()['amount_including_tax'];
        }

        return $totals;
    }

    public function getAdjustmentsTotalRecursivelyWithoutNeutrality(?string $type = null): int
    {
        $total = 0;
        foreach ($this->getAdjustmentsRecursively($type) as $adjustment) {
            $total += $adjustment->getAmount();
        }

        return $total;
    }
}
