<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Twig;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusEcotaxPlugin\Specification\AddressHasEcotaxSpecificationInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

class EcotaxDisplayExtension extends AbstractExtension
{
    public function __construct(
        private AddressHasEcotaxSpecificationInterface $addressHasEcotaxSpecification,
        private SettingsProviderInterface $settingsProvider,
    ) {
    }

    public function getFunctions()
    {
        return [
            new TwigFunction('dedi_ecotax_enabled', [$this, 'isEcotaxEnabled']),
            new TwigFunction('dedi_ecotax_order_has_ecotax', [$this, 'orderHasEcotax']),
        ];
    }

    public function isEcotaxEnabled(): bool
    {
        $settings = $this->settingsProvider->provide();

        return $settings->isEnabled();
    }

    public function orderHasEcotax(OrderInterface $order): bool
    {
        $address = $order->getShippingAddress();

        if (null === $address) {
            return false;
        }

        return $this->addressHasEcotaxSpecification->isSatisfiedBy($address);
    }
}
